--------------------------------------------  Default Textures  --------------------------------------------
local Statusbar_Texture = "Interface\\Addons\\TidyThreat\\Flat"
local Frame_Texture = "Interface\\Addons\\TidyThreat\\Thermo_Border"
local Glow_Texture = "Interface\\Addons\\TidyThreat\\Thermo_Glow"
local Tip_Font = "FONTS\\FRIZQT__.TTF"
-- <--------------------------------------------------------------------------------------------- *
TidyThreatDefaultOptions  = {
		Skin 				= 1, 
		Anchor				= "CENTER",
		PosX 				= -200,
		PosY 				= 200,
		Scale				= 1,
		AllowMove			= true,
		Visible				= true,
		ThreatLeadScale 	= 68,
		ShowDetailWindow	= false,
		HideOutOfCombat = false,
		PlayAlertSound = false,
}




--------------------------------------------  Configuration Variables  --------------------------------------------
local CLASS_HEX_COLOR = {
	DEATHKNIGHT = "C41F3B",
	DRUID = "FF7D0A",
	HUNTER = "ABD473",
	MAGE = "69CCF0",
	PALADIN = "F58CBA",
	PRIEST = "FFFFFF",
	ROGUE = "FFF569",
	SHAMAN = "2459FF",
	WARLOCK = "9482C9",
	WARRIOR = "C79C6E",
	PET = "24FF24",
}
-- <--------------------------------------------------------------------------------------------- *
local NUMBER_SUFFIX = {
	"st", "nd", "rd", "th", "th", "th", "th", "th", "th", 
}
-- <--------------------------------------------------------------------------------------------- *
local THREATWARNINGSOUND = "Sound\\Interface\\PVPFlagTakenMono.wav"
if "Alliance" == UnitFactionGroup("player") then THREATWARNINGSOUND = "Sound\\Interface\\PVPFlagTakenHordeMono.wav" end
--------------------------------------------  Utility Functions  --------------------------------------------
local function copytable(original)
	local duplicate = {}
	for key, value in pairs(original) do
		if type(value) == "table" then duplicate[key] = copytable(value)
		else duplicate[key] = value end
	end
	return duplicate
end
-- <--------------------------------------------------------------------------------------------- *
local function SetFrameSize(frame, width, height)
	frame:SetHeight(height)
	frame:SetWidth(width)
end
-- <--------------------------------------------------------------------------------------------- *
local function SetDefaultStatusBar(statusBar)
	statusBar:SetStatusBarTexture(Statusbar_Texture)
	statusBar:SetStatusBarColor(1, 0, 0)
	statusBar:SetMinMaxValues(0,100)
	statusBar:SetValue(100)
	statusBar:SetFrameStrata("LOW")
	statusBar:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 16});
	statusBar:SetBackdropColor(0,0,.4,.4);
	statusBar:SetOrientation("VERTICAL")
end
-- <--------------------------------------------------------------------------------------------- *
local function GetUnitThreatPercent(unitId)
	local d1, d2, threat, d3, d4 = UnitDetailedThreatSituation(unitId, "target")
	if not threat then threat = 0 end
	return threat
end
-- <--------------------------------------------------------------------------------------------- *
local function valueToString(value)
	if value >= 1000000 then return format('%.1fm', value / 1000000)
	elseif value >= 1000 then return format('%.1fk', value / 1000)
	else return value end
end
-- <--------------------------------------------------------------------------------------------- *
local function GetUnitThreatValue(unitId)
		if unitId == nil then return 0 end
		local d1, d2, d3, d4, threat = UnitDetailedThreatSituation(unitId, "target")
		if not threat then threat = 0 end
		return threat
end
-- <--------------------------------------------------------------------------------------------- *
local function RankToText(rank)
	local text
	local r = mod(rank,10)

	if rank == 1 then return "|cFFFF3300Tanking|r|n" 
	elseif rank == 2 then color = "|cFFFFDD00"
	else color = "|cFFAAFF00" end
	
	if r > 0 then
	   if rank > 10 and rank < 20 then text = "th" 
	   else text = NUMBER_SUFFIX[r] end
	else
	   text = "th"
	end

	return color..rank..text.."|r|n"
end
--------------------------------------------  Frame Construction  --------------------------------------------
TidyThreatWindow = CreateFrame("Frame", nil, UIParent)
TidyThreatOptions = copytable(TidyThreatDefaultOptions)
--------------------------------------------  Member Functions  --------------------------------------------
function TidyThreatWindow:SetThreatBars(player, counter)
	if not counter then counter = 0 end
	if not player then player = 0 end
	if player < 100 then counter = 0 
		self.Glow:Hide() 
	else counter = player - counter	
		self.Glow:Show() end
	
	if player > 80 then self.PlayerThreat:SetStatusBarColor(1, .36, 0)
	elseif player > 60 then self.PlayerThreat:SetStatusBarColor(1, .61, 0)
	else self.PlayerThreat:SetStatusBarColor(.85, 1, 0) end
	
	self.PlayerThreat:SetValue(player)
	self.CounterThreat:SetValue(counter)
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:SetThreatPopup(unitId, rank, playerThreat, groupThreat)
	
	local groupName = UnitName(unitId)
	local threatDifference = (playerThreat - groupThreat)/100
	local _, groupClass = UnitClass(unitId);
	
	local threatMessage = ""
	local hexColor = CLASS_HEX_COLOR[groupClass]
	if not hexColor then hexColor = "888888" end
	
	if playerThreat > 0 then
		if threatDifference > 0 then threatMessage = "|cFFFF3300+"..valueToString(threatDifference).."|r above |cFF"..hexColor..groupName
		elseif threatDifference < 0 then threatMessage = "|cFF33FF00"..valueToString(threatDifference).."|r below |cFF"..hexColor..groupName
		else threatMessage = "Your threat is equal to, |cFF"..hexColor.."0"..groupName.."'s" end
		rankMessage = RankToText(rank)
		threatMessage = rankMessage..threatMessage
	else
		threatMessage = nil
	end
	
	self:SetDetailText(threatMessage)
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:SetDetailText(message)
	if message and TidyThreatOptions.ShowDetailWindow then
		self.DetailWindow:Show()
		self.DetailWindow.Text:SetText(message)
		self.DetailWindow.Text:SetWidth(self.DetailWindow.Text:GetStringWidth())
		self.DetailWindow.Text:SetHeight(self.DetailWindow.Text:GetStringHeight())
	else
		self.DetailWindow:Hide()
	end
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:SetMoveable(allow)
	self:EnableMouse(allow)
	self:SetMovable(allow)
	if allow then
		self:SetScript("OnMouseDown", function(self) self:StartMoving() end)
		self:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
	else
		self:SetScript("OnMouseDown", nil)
		self:SetScript("OnMouseUp", nil)	
	end
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:AlertSound(threat)
	if threat > 80 then
		if not self.PlayerWarned then 
			PlaySoundFile(THREATWARNINGSOUND)
			self.PlayerWarned = true
		end
	else
		self.PlayerWarned = false
	end
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:SetVisible(visible)
	if visible then
		TidyThreatWindow:Show()
	else
		TidyThreatWindow:Hide()
	end
end
-- <--------------------------------------------------------------------------------------------- *

-- <--------------------------------------------------------------------------------------------- *

-- <--------------------------------------------------------------------------------------------- *
function GetThreatProperties(target)
		local playerThreat = 0
		local playerRank = 1
		local partyHighUnit = nil
		local partyThreat = 0
		local unit = nil
		local unitThreat = 0
		local partyThreatVal, playerThreatVal
		local fadedThreat
		
		-- Check Player's Threat
		playerThreat = GetUnitThreatPercent("player")
		
		-- Group Threat
		if HasPetUI() then					-- Check Pet's Threat
				unit = "pet"
				unitThreat = GetUnitThreatPercent("pet")
				if unitThreat > partyThreat then 
					partyThreat = unitThreat
					partyHighUnit = "pet"
					if unitThreat > playerThreat then playerRank = playerRank + 1 end
				end
		end
		if UnitInRaid("player") then 		-- Check in Raid
			local n = GetNumRaidMembers()
			for i = 1, n-1 do
				unitThreat = GetUnitThreatPercent("raid"..i)
				if unitThreat > partyThreat then 
					partyThreat = unitThreat
					partyHighUnit = "raid"..i
					if unitThreat > playerThreat then playerRank = playerRank + 1 end
				end
				
				unitThreat = GetUnitThreatPercent("raidpet"..i)
				if unitThreat > partyThreat then 
					partyThreat = unitThreat
					partyHighUnit = "raidpet"..i
					if unitThreat > playerThreat then playerRank = playerRank + 1 end
				end
			end 							-- Check in Party
		elseif UnitInParty("player") then 
			for i = 1, 4 do
				unitThreat = GetUnitThreatPercent("party"..i)
				if unitThreat > partyThreat then 
					partyThreat = unitThreat
					partyHighUnit = "party"..i
					if unitThreat > playerThreat then playerRank = playerRank + 1 end
				end

				unitThreat = GetUnitThreatPercent("partypet"..i)
				if unitThreat > partyThreat then 
					partyThreat = unitThreat
					partyHighUnit = "partypet"..i
					if unitThreat > playerThreat then playerRank = playerRank + 1 end
				end
			end 
		end
		
		if partyHighUnit then
			playerThreatVal = GetUnitThreatValue("player")
			partyThreatVal = GetUnitThreatValue(partyHighUnit)
			if playerThreatVal < 0 then 
				-- Corrected Val
				playerThreatVal = playerThreatVal + 410065408 
				-- Player Faded Threat
				fadedThreat = (playerThreatVal / partyThreatVal)*100
				if fadedThreat > 130 then 
					partyThreat = fadedThreat - 130 
					playerThreat = 100
				else 
					partyThreat = 0
					playerThreat = fadedThreat
				end
			end
		end
		
		return playerThreat, playerThreatVal, playerRank, partyThreat, partyThreatVal, partyHighUnit
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:OnUpdate()
	--if not self.Visible then return end
	if not TidyThreatWindow:IsShown() then return end
	
	if UnitAffectingCombat("player") then
		local playerThreat, playerThreatVal, playerRank, partyThreat, partyThreatVal, partyHighUnit = GetThreatProperties("target")
		
		if partyHighUnit then 
			self:SetAlpha(1)
			self:Show()
			self:SetThreatBars(playerThreat, partyThreat)
			if TidyThreatOptions.PlayAlertSound then self:AlertSound(playerThreat) end
			self:SetThreatPopup(partyHighUnit, playerRank, playerThreatVal, partyThreatVal) 
		else	
			--if TidyThreatOptions.HideOutOfCombat then self:Hide() end
			if TidyThreatOptions.HideOutOfCombat then self:SetAlpha(0) else  self:SetAlpha(1) end
			self:SetThreatBars(0, 0)
			self.DetailWindow:Hide() 
		end
		
	else
		--if TidyThreatOptions.HideOutOfCombat then self:Hide() end
		if TidyThreatOptions.HideOutOfCombat then self:SetAlpha(0) else self:SetAlpha(1) end
		self:SetThreatBars(0, 0)
		self.DetailWindow:Hide()
	end
end
-- <--------------------------------------------------------------------------------------------- *
function TidyThreatWindow:Startup()
	self.PlayerWarned = false
	
	self.PlayerThreat = CreateFrame("StatusBar", nil, self)
	self.CounterThreat = CreateFrame("StatusBar", nil, self)
	self.DetailWindow = CreateFrame("Frame",nil,self)
	self.DetailWindow.Text = self.DetailWindow:CreateFontString(nil, "ARTWORK")
	self.Border = self:CreateTexture(nil, "OVERLAY")
	self.Glow = self:CreateTexture(nil, "OVERLAY")
		
	-- Set the initial properties of the main (invisible) frame, and enable click/drag
	SetFrameSize(self, 35, 88)
	self:SetClampedToScreen()

	self:SetScript("OnEvent", self.OnEvent)
	self:RegisterEvent("PLAYER_LOGOUT") -- Fired when about to log out
	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_ATTACK")
	
	-- Set the initial properties of the Player Threat Bar
	SetFrameSize(self.PlayerThreat, 20, 53)
	SetDefaultStatusBar(self.PlayerThreat)
	self.PlayerThreat:SetPoint("BOTTOM", self, "BOTTOM", 0, 8)
	self.PlayerThreat:SetMinMaxValues(0,100)
	
	-- Set the initial properties of the Threat Lead Extension Bar
	SetFrameSize(self.CounterThreat, 20, 18)
	SetDefaultStatusBar(self.CounterThreat)
	self.CounterThreat:SetPoint("BOTTOM", self.PlayerThreat, "TOP" )
	self.CounterThreat:SetMinMaxValues(0, 68)
	
	-- Set the initial properties of the Overlay 
	SetFrameSize(self.Border, 64, 128)
	self.Border:SetTexture(Frame_Texture)
	self.Border:SetPoint("CENTER", 0, 3)
	
	-- Set the initial properties of the Threat Glow
	self.Glow:SetAllPoints(self.Border)
	self.Glow:SetTexture(Glow_Texture)
	self.Glow:SetVertexColor(1, 0, 0);
	self.Glow:Hide()	
	
	-- Set the initial properties of the Detail popup
	self.DetailWindow:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	self.DetailWindow:SetBackdropColor(0,0,0,.75);
	self.DetailWindow:SetPoint("LEFT", self.DetailWindow.Text, "LEFT", -8, 0)
	self.DetailWindow:SetPoint("RIGHT", self.DetailWindow.Text, "RIGHT", 8, 0)
	self.DetailWindow:SetPoint("TOP", self.DetailWindow.Text, "TOP", 0, 7)
	self.DetailWindow:SetPoint("BOTTOM", self.DetailWindow.Text, "BOTTOM", 0, -8)
	
	self.DetailWindow:SetPoint("CENTER", self.DetailWindow.Text, "CENTER")
	self.DetailWindow.Text:SetFont(Tip_Font, 10)
	self.DetailWindow.Text:SetPoint("TOP", self, "BOTTOM", 0, -15)
	self.DetailWindow.Text:SetJustifyH("CENTER")
	self.DetailWindow.Text:SetJustifyV("CENTER")
	self:SetDetailText(nil)
end
--------------------------------------------  Mouseover Threat  --------------------------------------------
local HexReddy = "|cFFFF3300"
local HexAmber = "|cFF33FF00"
local HexGreen = "|cFF8DC63F"
local HexWhite = "|cFFFFFFFF"

local function round(what, precision)  
   return math.floor(what*math.pow(10,precision)+0.5) / math.pow(10,precision)  
end 

GameTooltip:HookScript("OnTooltipSetUnit", function(self) 
	local name, id = GameTooltip:GetUnit()
	if InCombatLockdown() and id then
		if id == "mouseover" or id == "focus" or id == "target" then
			local messageLeft, messageRight
			local threatpct, threatval, rank, pthreatpct, pthreatval, pname = GetThreatProperties(id)
			if threatval and pthreatval then
				local diff = (threatval - pthreatval)/100
				local percent = threatpct+(100-pthreatpct)
				
				if threatpct == 100 then messageLeft = HexReddy.."Tanking |r("..round(percent,1).."%|r)"					
				else messageLeft = HexGreen.."Rank "..rank.." |r("..round(percent,1).."%|r)" end

				if diff > 0 then messageRight = HexReddy.."+"..round(diff,1)..HexWhite.." above "..UnitName(pname)
				elseif diff < 0 then messageRight = HexGreen..""..round(diff,1)..HexWhite.." below "..UnitName(pname)
				else messageRight = "|rThreat is equal" end

				self:AddDoubleLine(messageLeft, messageRight)
				self:Show()
			end
		end
	end
end)
--------------------------------------------  Events  --------------------------------------------
function TidyThreatWindow:OnEvent(event)
	if event == "PLAYER_LOGOUT" then 
		-- Save Position
		TidyThreatOptions.Anchor, _, _, TidyThreatOptions.PosX, TidyThreatOptions.PosY = self:GetPoint()
	elseif event == "VARIABLES_LOADED" then
	
		-- Load Position
		self:ClearAllPoints()
		self:SetPoint(TidyThreatOptions.Anchor, TidyThreatOptions.PosX, TidyThreatOptions.PosY)	
		-- Apply Configuration
		if type(self.ApplyOptions) == 'function' then self:ApplyOptions() end
	elseif event == "PLAYER_TARGET_CHANGED" then self.PlayerWarned = false end
	self:OnUpdate()
end
--------------------------------------------  Slash Commands  --------------------------------------------
TidyThreatCommands = {}
TidyThreatCommands.hide = function() TidyThreatOptions.Visible = false; TidyThreatOptions.HideOutOfCombat = false;   end
TidyThreatCommands.show = function() TidyThreatOptions.Visible = true; TidyThreatOptions.HideOutOfCombat = false;   end
TidyThreatCommands.lock = function() TidyThreatOptions.AllowMove = false;   end
TidyThreatCommands.unlock = function() TidyThreatOptions.AllowMove = true;   end
TidyThreatCommands.detail = function() TidyThreatOptions.ShowDetailWindow = true;   end
TidyThreatCommands.nodetail = function() TidyThreatOptions.ShowDetailWindow = false;   end
TidyThreatCommands.autohide = function() TidyThreatOptions.Visible = true; TidyThreatOptions.HideOutOfCombat = true;   end
TidyThreatCommands.sound = function() TidyThreatOptions.PlayAlertSound = true;   end
TidyThreatCommands.nosound = function() TidyThreatOptions.PlayAlertSound = false;   end
TidyThreatCommands.reset = function() TidyThreatOptions = copytable(TidyThreatDefaultOptions)  end

--------------------------------------------  Slash Commands  --------------------------------------------
function slash_TidyThreat(arg)
	if type(TidyThreatCommands[arg]) == 'function' then TidyThreatCommands[arg](); TidyThreatWindow:ApplyOptions() else
	ChatFrame1:AddMessage('TidyThreat; Usage: /tidythreat or /ttt [show, hide, lock, unlock, detail, nodetail, sound, nosound, reset, autohide]');end
end

SLASH_TIDYTHREAT1 = '/tidythreat'
SLASH_TIDYTHREAT2 = '/ttt'
SlashCmdList['TIDYTHREAT'] = slash_TidyThreat;

TidyThreatWindow:Startup()
